%% Compressed Diffusion LMS

% Authors: Ibrahim El khalil Harrane, R?my Flamary, C?dric Richard
% Please cite the paper refrenced below if you use this code.
% This code corresponds to the second experimentation of the paper
% (Performance comparison for both CDLMS and DCDLMS).

% @article{harranedoubly,
%   title={Doubly compressed diffusion LMS over adaptive networks},
%   author={Harrane, Ibrahim El Khalil and Flamary, R{\'e}mi and Richard, C{\'e}dric}

%%
clear all
close all
addpath(genpath('.'));
addpath(genpath('../toolbox'));

%% Simulation settings

rng(356);
L=50;                                              % w_o length
N=50;                                              % number of nodes
sel=[5 5 5; 10 10 10; 20 20 13;30 40 26 ];         % [M M M_{\nabla} ] the parameters M and M_{\nabla} for both algorithms
N_iter=1.2e5;                                      % number of iteration
N_exp=100;                                           % number of experimentations
step_size=1e-3;                                    % step size
mu=step_size*ones(N,1);
sigma_u=1e1;                                       % regression vector variation
Sigma_u=rand(N,1)+1;
sigma_v=1e-3;                                      % noise variation
Sigma_v=sigma_v*ones(N,1);
r=1.5;                                             % distance parameter for network topology
w_star=randn(L,1);                                 % desired parameter selection
w_0=repmat(randn(L,1),1,N);                        % local estimates initialisation

%%


options.M=L;
options.N=N;
options.N_iter=N_iter;
options.parameter=r;
options.corr=0;
options.type='radial';
options.wo=w_star;
options.rule='metropolis';
options.sigma_u=sigma_u;
options.Sigma_u=Sigma_u;
options.sigma_v=sigma_v;
options.one_iter=0;
options.rtype='provided';
load('seed_10.mat');

%%

rng(seed)
[X,L1,Ac,xy] = get_network( options);
[ Coef ] = get_coef( X,Sigma_u,Sigma_v,mu,options );
C=Coef;
A=eye(N);




%% Monte-Carlo runs
MSD_cmp=[];
MSD_d_cmp=[];

for k=1:size(sel,1)
    
    fprintf('sim n %d  \n',k);
    e_compressed=zeros(N_iter,N);
    e_doubly_compressed=zeros(N_iter,N);
    
    
    
    
    M=sel(k,1);
    M_d=sel(k,2);
    M_grad=sel(k,3);
    
    rng(356);
    
    for l=1:N_exp
        
        [ d,u,Sigma_u,Sigma_v,w_star] = get_data( options );
        u=permute(u,[2 3 1]);
        
        H=zeros(L,N,N_iter);
        Q=zeros(L,N,N_iter);
        
        for i=1:N
            for j=1:N_iter
                H(:,i,j)=randperm(L);
                Q(:,i,j)=randperm(L);
            end
        end
        
        
        
        fprintf('exp n %d  \n',l);
        
        [e_t,w_k_compressed]=compressed_diffusion(A,C,w_0,u,d,mu,H,M,w_star);
        e_compressed=e_compressed+e_t;
        
        [e_t1,w_k_doubly_compressed]=doubly_compressed_diffusion(A,C,w_0,u,d,mu,H,Q,M_d,M_grad,w_star);
        e_doubly_compressed=e_doubly_compressed+e_t1;
    end
    
    MSD_cmp=[MSD_cmp mean(e_compressed,2)/N_exp];
    MSD_d_cmp=[MSD_d_cmp mean(e_doubly_compressed,2)/N_exp];
    
end
%% MSD curves





figure(1)
clf
set(0, 'defaultTextInterpreter', 'latex')
fs=20;



h=plot(10*log10(MSD_cmp));

grid on


l=legend('$ r = 1.81 $','$ r = 1.66 $',' $ r = 1.43 $',' $ r = 1.25 $');
set(l,'Interpreter', 'latex','FontSize',0.8*fs)
hl=findobj(gcf,'type','axes','tag','legend');

Pos=get(hl,'position');
set(hl,'Position',Pos)

title({'Compressed diffusion LMS'},...
    'Interpreter','latex','FontSize',1.2*fs)
axis([0 6e4 -60 20]);
xlabel(' iteration $ i $ ','Interpreter', 'latex','FontSize',1*fs)
ylabel('MSD (dB)','Interpreter', 'latex','FontSize',1*fs)

set(gca,'Ytick',-70:10:0);
set(gca,'FontSize',0.8*fs)






set(gca,'FontSize',0.8*fs)
set(h(1:end),'Linewidth',0.5);

%%
figure(2)
clf
h=plot(10*log10(MSD_d_cmp));

grid on


l=legend('$ r = 10 $','$ r = 5 $',' $ r = 3 $',' $ r = 1.5 $');
set(l,'Interpreter', 'latex','FontSize',0.8*fs)
hl1=findobj(gcf,'type','axes','tag','legend');

title({'Doubly-compressed diffusion LMS  '},...
    'Interpreter','latex','FontSize',1.2*fs)
axis([0 N_iter -60 20]);
xlabel(' iteration $ i $ ','Interpreter', 'latex','FontSize',1*fs)
ylabel('MSD (dB)','Interpreter', 'latex','FontSize',1*fs)

set(gca,'Ytick',-70:10:0);
set(gca,'FontSize',0.8*fs)





set(gca,'FontSize',0.8*fs)
set(h(1:end),'Linewidth',0.5);




